/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Millipede class


*******************************************************************************
******************************************************************************/

#include "Millipede.h"

#define MIN_X ( -32 << 12 )
#define MIN_Y ( -18 << 12 )

#define MILLIPEDE_MAX_HP (464 * 4)

//------------------------------------------------------------------------------
//	Millipede Enemy
//------------------------------------------------------------------------------
Millipede::Millipede()
{
	x = 600 << 12;
	y = 96 << 12;
	Frame = 0;
	Active = false;	
	Alive = true;
	DestroyedCounter = 60 * 5;
	Hp = 256 * 4; 
	DrawMode = EN_NORMAL;
	FlipMode = GL_FLIP_NONE;
	
	ResetSegments();
	
}

Millipede::~Millipede()
{
}


void Millipede::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
}

void Millipede::Update()
{
	if( Active )
	{
		Counter++;
			
		// Update spline 
		// and get out of loop when we get 
		// a greater linear distance than 10
		s32 DistFromLast = 0;
		while( DistFromLast < floattof32(10) )
		{
	
	
			Path.Update();
			
			if( Path.GetIndex() == 5 ) ClosePath();
			
			x = Path.GetX();
			y = Path.GetY();
			
			// calculate distance of current point from last used point
			s32 dx = (( x - LastPoint.x ) * ( x - LastPoint.x )) >> 12;
			s32 dy = (( y - LastPoint.y ) * ( y - LastPoint.y )) >> 12;
			s32 d = sqrtf32( dx + dy );
			
			// increment linear distance
			DistFromLast += d;
		
		} // wend 
		
		// save point for use next loop
		LastPoint.x = x;
		LastPoint.y = y;
		
		// set up our centipede
		Segments[MILLIPEDE_MAX_SEGMENTS - 1].x = x;
		Segments[MILLIPEDE_MAX_SEGMENTS - 1].y = y;

		for( int i = 0; i < (MILLIPEDE_MAX_SEGMENTS - 1);  i++ )
		{
			Segments[i].x = Segments[i+1].x;
			Segments[i].y = Segments[i+1].y;
		}
		
	
	}  // active
	
	if( Active ) Targets::Instance()->AddCoord( x, y );
	
}


void Millipede::LoadPath( int _sx, int _sy, int _ex, int _ey )
{
	Path.Init( 24 );
	
	Spline::Coord p;
	
	// start
	p.x = _sx;
	p.y = _sy;
	
	// we should use 2 points
	Path.AddPoint( p );
	Path.AddPoint( p );
	
	// randomize for now
	for (int i=0; i < 16; i++)
	{
		p.x = qran_range(0, 255);
		p.y = qran_range(0, 191);
		Path.AddPoint( p );
	}
	
		p.x = 129;
		p.y = 96;
		Path.AddPoint( p );
	
		p.x = 20;
		p.y = 96;
		Path.AddPoint( p );
	
	// close last 2 coords
	p.x = _ex;
	p.y = _ey;
	
	// we should use 2 points
	Path.AddPoint( p );
	Path.AddPoint( p );
	
	Path.SetSpeed(8);
	Path.SetT(0);
	
}


void Millipede::Draw()
{
	glColor( RGB15(31,31,31) );
	if( Active )
	{
		if( (DrawMode == EN_WHITE) )
			{
				DrawMode = EN_NORMAL;
				glColor( RGB15(31,0,0) );
			
				for( int i = 0; i < MILLIPEDE_MAX_SEGMENTS; i+= 8 )
				{
					
					s32 tx = Segments[i].x;
					s32 ty = Segments[i].y;
					
					s32 angle;
					if( i == 0 )    //tail
					{
						angle = BRAD_PI + atan2Lerp( tx - Segments[i+7].x, 
													 ty - Segments[i+7].y 
												   );				
						Frame = 0;					
					}	
					else if( i == (MILLIPEDE_MAX_SEGMENTS-8) )	// head
					{
						angle = BRAD_PI + atan2Lerp( Segments[i-7].x - tx, 
													 Segments[i-7].y - ty 
												   );
						Frame = 0;
					}
					else	// segments
					{
						angle = BRAD_PI + atan2Lerp( Segments[i-7].x - tx, 
													 Segments[i-7].y - ty 
												   );
						Frame = 1;
					}
					
					
					glSpriteRotate( tx >> 12,
									ty >> 12,
									angle,
									GL_FLIP_NONE, &Images[Frame] 
								  );
							
				}  //next
				
				glColor( RGB15(31,31,31) );
					
			}   
			else	//draw normal
			{
				for( int i = 0; i < MILLIPEDE_MAX_SEGMENTS; i+= 8 )
				{
					
					s32 tx = Segments[i].x;
					s32 ty = Segments[i].y;
					
					s32 angle;
					if( i == 0 )    //tail
					{
						angle = BRAD_PI + atan2Lerp( tx - Segments[i+7].x, 
													 ty - Segments[i+7].y 
												   );				
						Frame = 0;					
					}	
					else if( i == (MILLIPEDE_MAX_SEGMENTS-8) )	// head
					{
						angle = BRAD_PI + atan2Lerp( Segments[i-7].x - tx, 
													 Segments[i-7].y - ty 
												   );
						Frame = 0;
					}
					else	// segments
					{
						angle = BRAD_PI + atan2Lerp( Segments[i-7].x - tx, 
													 Segments[i-7].y - ty 
												   );
						Frame = 1;
					}
					
					
					glSpriteRotate( tx >> 12,
									ty >> 12,
									angle,
									GL_FLIP_NONE, &Images[Frame] 
								  );
							
				}  // next
				
			} // endif color mode
		
	}   // active


	// draw energy
	int xx = (x >> 12) - 32;
	int yy = (y >> 12) - 35;
	
	int Length = 64;
	s32 Gauge = (Hp << 12) / MILLIPEDE_MAX_HP;
	int GaugeLength = ( UTIL::Lerpf32( 0, Length << 12, Gauge ) );
	glBoxFilledGradient( xx, yy,
						xx + (GaugeLength >> 12), 
						yy+ 4,
						RGB15(31,0,0),
						RGB15(31,0,0),
						RGB15(0,31,0),
						RGB15(0,0,31) );
	
	glBox(xx, yy, xx + (Length), yy + 4, RGB15(Counter, Counter/2, 31-Counter) );
	
}


bool Millipede::Collide( StarFighter &Star )
{
	
	if( Active )
	{
		
		for( int i = 0; i < MILLIPEDE_MAX_SEGMENTS; i+= 8 )
		{
			int rad2 = 12 * 12;
			int cx = (Segments[i].x >> 12) - (Star.x >> 12);
			int cy = (Segments[i].y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (2+Star.IsBursting())) )
			{
				if( Star.IsBursting() )
				{
					Vector2df32 p;
					p.x = Star.x;
					p.y = Star.y;
					Particle::Instance()->Spawn( p, 2 );
					Star.IncreaseGauge(10);
					Hp -= 4;
					if( (Counter & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					if( Hp < 0 )
					{
						Active = false;
						Alive = false;
						Star.Score += 50975;
						Sound::Instance()->PlaySFX(E_SFX_MEDEXP2);
						// Add so.me nice explosions
						for( int j = 0; j < MILLIPEDE_MAX_SEGMENTS; j+= 8 )
						{
							s32 dx = cosLerp(j * 128)*2;
							s32 dy = sinLerp(j * 256)*2;
							Explosion::Instance()->Spawn( Segments[j].x >> 12,
													  Segments[j].y >> 12,
													   dx,
													   dy,
													   Explode::MEDIUM_5,
													   Explode::MEDIUM_2
													 );
						}
						
						
						Explosion::Instance()->MegaSpawn(x >> 12,96,80,Explode::TINY,Explode::MEDIUM_5, 60*4 );
	
						x = 600 << 12;
						y = 96 << 12;
						
					}
					if( DrawMode == EN_NORMAL )
					{
						if( (Counter & 1) ) DrawMode = EN_WHITE;
					}
				}
				else
				{
					// kill star
					if( !Star.IsInvincible() ) Star.Kill();
				}
				
				return true;
			}	
		}  // next segs
				    
	} //active
	
	
	// missiles if after star enemy is still active
	if( Active )
	{
		
		// Missiles
		for( int k = 0; k < MAX_MISSILES; k++)
		{
			HomingMissile Missile= Star.GetMissile(k);
			
			if( Missile.IsActive() )
			{
				
				for( int i = 0; i < MILLIPEDE_MAX_SEGMENTS; i+= 8 )
				{
					int rad2 = 16 * 16;
				
					if( Segments[i].x < ( 256 << 12 ) )
					{
			
						int cx = (Segments[i].x >> 12) - (Missile.GetX() >> 12);
						int cy = (Segments[i].y >> 12) - (Missile.GetY() >> 12);
						if( ( (cx * cx) + (cy * cy) ) < (rad2) )
						{
							Vector2df32 p;
							p.x = Missile.GetX();
							p.y = Missile.GetY();
							Particle::Instance()->Spawn( p, 1 );
							if( (Counter & 1) ) DrawMode = EN_WHITE;
							Star.DestroyMissile(k);
							Hp--;
							if( (Counter & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					
							if( Hp < 0 )
							{
								Active = false;
								Alive = false;
								Star.Score += 50975;
								Sound::Instance()->PlaySFX(E_SFX_MEDEXP2);
								// Add some nice explosions
								for( int j = 0; j < MILLIPEDE_MAX_SEGMENTS; j+= 8 )
								{
									s32 dx = cosLerp(j * 128)*2;
									s32 dy = sinLerp(j * 256)*2;
									Explosion::Instance()->Spawn( Segments[j].x >> 12,
															  Segments[j].y >> 12,
															   dx,
															   dy,
															   Explode::MEDIUM_5,
															   Explode::MEDIUM_2
															 );
								}
								
								Explosion::Instance()->MegaSpawn(x >> 12,96,80,Explode::TINY,Explode::MEDIUM_5, 60*4 );
	
								x = 600 << 12;
								y = 96 << 12;
								break;
							}
						}
				
					}
				}
			
			}   //  x < 
					
		} // segments	
		
	}   //active
		

	return false;
}



void Millipede::Spawn()
{
	
	Active = true;
	Alive = true;	
	DestroyedCounter = 60 * 5;
	Frame = 0;
	Counter = 0;
	Hp = 464 * 4;
	LoadPath( 320, 96, -300, 96 );

	
}

void Millipede::ClosePath()
{
		
	Path.SetPoint( 0, qran_range(0, 255), qran_range(0, 193) );
	Path.SetPoint( 1, qran_range(0, 255), qran_range(0, 193) );
	
	Path.SetPoint( Path.GetMaxPoints() - 1, qran_range(0, 255), qran_range(0, 193) );
	Path.SetPoint( Path.GetMaxPoints() - 2, qran_range(0, 255), qran_range(0, 193) );
	
}

