/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Tripod class


*******************************************************************************
******************************************************************************/

#include "Tripod.h"

#define MIN_X ( -20 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_TRIPODS 16

//------------------------------------------------------------------------------
//	Tripod Enemy
//------------------------------------------------------------------------------
Tripod::Tripod()
{
	Frame = 0;
	Active = false;	
	Hp = 1; 
	DrawMode = EN_NORMAL;
	
}

Tripod::~Tripod()
{
}


void Tripod::Update()
{

	
	Counter++;
	
	if (Counter == 60 * 20)
	{
		Target.x = -400 << 12;
		Target.y = qran_range(0,191) << 12;
		NewTarget = false;
		TargetMe = false;
	}
	
	if( !TargetMe )
	{
		if (NewTarget)
		{
			Target.x = qran_range(0,255) << 12;
			Target.y = qran_range(0,191) << 12;
			NewTarget = false;
			TargetMe = false;
		}
	}
	
	s32 dx = mulf32(cosLerp(Angle), Speed);
	s32 dy = mulf32(sinLerp(Angle), Speed);
	
	x += dx;
	y += dy;
	
	// distance
	if ( (abs(x - Target.x) + abs(y - Target.y)) < (16 << 12) )
            NewTarget = true;
	
	// Project normal
	s32 dot = (mulf32(dy , (Target.x - x)) - mulf32(dx, (Target.y - y)));
	
	if( dot > 0)
		Angle = (Angle - Delta);
	else
		Angle = (Angle + Delta);
	
	        
	if( x < MIN_X )
	{
		Active = false;
		x = 300 << 12;
		y = 96 << 12;
	}

	// animate
	if( ( Counter & 3 ) == 0 )
	{
		Frame = (Frame + 1) & 3;
	}
	
	if( Active ) Targets::Instance()->AddCoord( x, y );

}


	
//------------------------------------------------------------------------------
//	Tripod Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
TripodContainer::TripodContainer()
{
}
	
TripodContainer::~TripodContainer()
{
	Tripods.clear();				// clear the bullets
}


void TripodContainer::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Tripods.resize( MAX_TRIPODS/2 );			// 1/2 of max (speed of iteration)
	Tripods.reserve( MAX_TRIPODS );		// reserve max for speed.  No need to do this for enemies
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void TripodContainer::UpdateEntities()
{
	
	std::vector<Tripod>::iterator iter;
	for( iter = Tripods.begin(); iter != Tripods.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void TripodContainer::DrawEntities()
{
	
	glColor( RGB15(31,31,31) );
	numEntities = 0;
	std::vector<Tripod>::iterator iter;
	for( iter = Tripods.begin(); iter != Tripods.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColor( RGB15(31,0,0) );
				glSpriteRotate( (iter->x >> 12), (iter->y >> 12), 
					      BRAD_PI/2+iter->Angle,GL_FLIP_NONE, &Images[iter->Frame] 
					    );
				glColor( RGB15(31,31,31) );
	
			}
			else
			{
				glSpriteRotate( (iter->x >> 12), (iter->y >> 12), 
						  BRAD_PI/2+iter->Angle,GL_FLIP_NONE, &Images[iter->Frame] 
					    );
			}
			
		}
	}
	
}

bool TripodContainer::Collide( StarFighter &Star )
{
	std::vector<Tripod>::iterator iter;
	for( iter = Tripods.begin(); iter != Tripods.end(); ++iter )
	{
		if( iter->Active )
		{
			// targetme
			if( iter->TargetMe )
			{
				iter->Target.x = Star.x;
				iter->Target.y = Star.y;
			}
			if( ((iter->Counter & 255) == 0) && !iter->TargetMe )
			{
				iter->TargetMe = true; //!iter->TargetMe;
				iter->NewTarget = false;
				//if( iter->TargetMe )
				//{
					iter->Target.x = Star.x;
					iter->Target.y = Star.y;
					iter->Speed += 4096;
					iter->Delta += 512;
				//}
			}
			int rad2 = 8 * 8;
			int cx = (iter->x >> 12) - (Star.x >> 12);
			int cy = (iter->y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
			{
				if( Star.IsBursting() )
				{
					Vector2df32 p;
					p.x = Star.x;
					p.y = Star.y;
					Particle::Instance()->Spawn( p, 2 );
					Star.IncreaseGauge(15);
					iter->Hp--;
					if( (iter->Hp & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					if( iter->Hp < 0 )
					{
						iter->Active = false;
						Star.Score += 33;
						Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
						// Add some nice explosions
						Explosion::Instance()->Spawn( iter->x >> 12,
													  iter->y >> 12,
													   0,
													   0,
													   Explode::SMALL_1
													 );
							
						iter->x = 600 <<12;
						iter->y = 96 << 12;
						
					}
					if( iter->DrawMode == EN_NORMAL )
					{
						if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
					}
				}
				else
				{
					// kill star
					if( !Star.IsInvincible() ) Star.Kill();
				}
				return true;
			}
		}
		
		// missiles if after star enemy is still active
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			
			if( iter->x < ( 256 << 12 ) )
			{
			
				// Missiles
				for( int i = 0; i < MAX_MISSILES; i++)
				{
					HomingMissile Missile= Star.GetMissile(i);
					
					if( Missile.IsActive() )
					{
						int cx = (iter->x >> 12) - (Missile.GetX() >> 12);
						int cy = (iter->y >> 12) - (Missile.GetY() >> 12);
						if( ( (cx * cx) + (cy * cy) ) < (rad2) )
						{
							Vector2df32 p;
							p.x = Missile.GetX();
							p.y = Missile.GetY();
							Particle::Instance()->Spawn( p, 1 );
							Star.DestroyMissile(i);
							iter->Hp--;
							if( iter->Hp < 0 )
							{
								iter->Active = false;
								Star.Score += 33;
								Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
								// Add some nice explosions
								Explosion::Instance()->Spawn( iter->x >> 12,
															  iter->y >> 12,
															   0,
															   0,
															   Explode::SMALL_4
															 );
									
								iter->x = 600 <<12;
								iter->y = 96 << 12;
								break;
							}
						}
				
					}
				}
			
			}   // iter x < 
	
		}   // iter active
		
	}
	return false;
}



void TripodContainer::KillAllEntities()
{
	std::vector<Tripod>::iterator iter;
	for( iter = Tripods.begin(); iter != Tripods.end(); ++iter )
	{
		if( iter->Active )
		{
			iter->Active = false;
			iter->x = 600 << 12;
			iter->y = 96 << 12;
		}
	}
	
}

//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void TripodContainer::Spawn( int _x, int _y )
{
	
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Tripod>::iterator iter;
	for( iter = Tripods.begin(); iter != Tripods.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{

			iter->x = _x << 12;
			iter->y = _y << 12;
			iter->Speed = floattof32(2);
			iter->Angle = 0;
			iter->Delta = 256;
			iter->Target.x = qran_range(0,255) << 12;
			iter->Target.y = qran_range(0,191) << 12;
			iter->NewTarget = true;
			iter->TargetMe = false;
		
			iter->Active = true;
			iter->Frame = 0;
			iter->Counter = 0;
			iter->Hp = 1;
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Tripods.size() < MAX_TRIPODS) )
	{
		Tripod popcorn;
		popcorn.x = _x << 12;
		popcorn.y = _y << 12;
		popcorn.Speed = floattof32(2);
		popcorn.Angle = 0;
		popcorn.Delta = 256;
		popcorn.Target.x = qran_range(0,255) << 12;
		popcorn.Target.y = qran_range(0,191) << 12;
		popcorn.NewTarget = true;
		popcorn.TargetMe = false;
		
		popcorn.Active = true;
		popcorn.Frame = 0;
		popcorn.Counter = 0;
		popcorn.Hp = 1;
		Tripods.push_back(popcorn);
	}
	
}




